<?php

namespace backend\forms;

use common\logic\Amqp\Cache;
use common\logic\Amqp\Message;
use components\Exception;

class MessageForm extends BaseForm
{
	public $email;

	public $password;

	public $type;

	public $send_arr;

	private $_queue = '';

	/**
	 * 登录方式
	 */
	const TYPE_SMS	 = 'sms';

	const TYPE_EMAIL = 'email';

	public function rules()
	{
		return [
			//login
			//[['email', 'password', 'type'], 'safe', 'on' => 'login'],
			[['email', 'password', 'type'], 'trim', 'on' => 'login'],
			[['email', 'password'], 'required', 'on' => 'login'],
			['email', 'email', 'on' => 'login'],
			['type', 'default', 'value' => static::TYPE_EMAIL, 'on' => 'login'],
			['type', 'in', 'range' => [static::TYPE_EMAIL, static::TYPE_SMS], 'on' => 'login'],
			//logins
			[['send_arr'], 'required', 'on' => 'logins']
		];
	}

	public function setQueue($scenario)
	{
		$this->_queue = $scenario;
	}

	public function getQueue()
	{
		return $this->_queue;
	}

	/**
	 * [发送消息]
	 * @author: libingke
	 * @return array
	 * @throws Exception
	 */
	public function sendMessage()
	{
		$body = json_encode([
			'email'		=> $this->email,
			'password'	=> $this->password,
			'type'		=> $this->type
		]);
		$queue = $this->getQueue();

		$data = [];
		try {
			$message = new Message($queue);
			$corrId = $message->send($body, $queue);

			Cache::setData($corrId, Cache::STATUS_SEND_OK);	//set: status = send ok

			$data[] = $corrId;

		} catch (\common\logic\Amqp\Exception $e) {
			throw new Exception($e->getCode(), $e->getMessage());
		}

		return $data;
	}
}